package nl.amis.adfbc.services;

import java.util.HashMap;
import java.util.List;

import nl.amis.webservice.proxy.CountryInfoService;
import nl.amis.webservice.proxy.CountryInfoServiceSoapType;
import nl.amis.webservice.proxy.types.ArrayOftCountryCodeAndName;
import nl.amis.webservice.proxy.types.TCountryCodeAndName;

import oracle.jbo.ApplicationModule;
import oracle.jbo.Row;
import oracle.jbo.ViewObject;
import oracle.jbo.client.Configuration;


public class BatchJobSynchronizeCountries {
  private static CountryInfoService countryInfoService;
  
    public static void main(String[] args) {
        String amDef = "nl.amis.adfbc.services.AppModule";
        String config = "AppModuleLocal";
        ApplicationModule am =
            Configuration.createRootApplicationModule(amDef, config);
        ViewObject countryVw = am.findViewObject("CountryVw");
        
        // Work with your appmodule and view object here
        System.out.println("Number of countries in CountryVw: "+countryVw.getEstimatedRowCount());
        
        //Get generated webservice proxy classes form the client project
        countryInfoService = new CountryInfoService(); 
        CountryInfoServiceSoapType countryInfoServiceSoapType = countryInfoService.getCountryInfoServiceSoap();
        
        //1- First update existing country rows in the COUNTRIES table
        countryVw.executeQuery();
        int rownr=0;
        while (countryVw.hasNext()) {
            rownr++;
            Row curCountry = countryVw.next();
            String isoCountryCode = (String)curCountry.getAttribute("CountryId");
            System.out.println("--- UPDATING (" + rownr + ")\t isoCountryCode: " + isoCountryCode+" country: "+countryInfoServiceSoapType.countryName(isoCountryCode));      
            
            //Put attributes on your ViewObject row
            curCountry.setAttribute("Capital", countryInfoServiceSoapType.capitalCity(isoCountryCode));
            curCountry.setAttribute("Intphonecode", countryInfoServiceSoapType.countryIntPhoneCode(isoCountryCode));
            curCountry.setAttribute("Countryflag", countryInfoServiceSoapType.countryFlag(isoCountryCode));
            curCountry.setAttribute("Languagename", countryInfoServiceSoapType.languageName(isoCountryCode));
            curCountry.setAttribute("CountryName", countryInfoServiceSoapType.countryName(isoCountryCode));
        }
        
        //Query all countries and put them in a HashMap. 
        //Purpose: separate an UPDATE and an INSERT in COUNTRIES table
        countryVw.executeQuery();     
        HashMap landCodesMap = new HashMap();
        while (countryVw.hasNext()) {            
            Row curCountry = countryVw.next();
            landCodesMap.put(curCountry.getAttribute("CountryId"),curCountry.getAttribute("CountryName"));
        }
        
        //2- Insert new country rows in the COUNTRIES table
        //Get all countrycodes from the webservice
        ArrayOftCountryCodeAndName arrayOftCountryCodeAndName = countryInfoServiceSoapType.listOfCountryNamesByCode();
        List<TCountryCodeAndName> tCountryCodeAndNameList = arrayOftCountryCodeAndName.getTCountryCodeAndName();
        for(TCountryCodeAndName tCountryCodeAndName:tCountryCodeAndNameList){
            
          String isoCountryCode = tCountryCodeAndName.getSISOCode();                                      
          //Add it country to table COUNTRIES when it does not yet exists
          if (!landCodesMap.containsKey(tCountryCodeAndName.getSISOCode())) {
                System.out.println("--- INSERTING IN COUNTRIES TABLE  isoCountryCode: "+tCountryCodeAndName.getSISOCode()+" - country: "+tCountryCodeAndName.getSName());
                // Create new row and add it to the rowset
                Row newCounrtyRow = countryVw.createRow();
                countryVw.insertRow(newCounrtyRow);

                //KOLOM DDBEGIN_INKOMSTENVERHOUDING
                newCounrtyRow.setAttribute("CountryId", tCountryCodeAndName.getSISOCode());
                newCounrtyRow.setAttribute("Capital", countryInfoServiceSoapType.capitalCity(isoCountryCode));
                newCounrtyRow.setAttribute("Intphonecode", countryInfoServiceSoapType.countryIntPhoneCode(isoCountryCode));
                newCounrtyRow.setAttribute("Countryflag", countryInfoServiceSoapType.countryFlag(isoCountryCode));
                newCounrtyRow.setAttribute("Languagename", countryInfoServiceSoapType.languageName(isoCountryCode));
                newCounrtyRow.setAttribute("CountryName", countryInfoServiceSoapType.countryName(isoCountryCode));
              } 
            }   
            
        //commit transaction
        am.getTransaction().commit();
        System.out.println("Country-batch SUCCESFULLY synchronized the COUNTRIES table with webservice"); 
        Configuration.releaseRootApplicationModule(am, true);
    }
}